/* ==========================================
 * jORLib : a free Java OR library
 * ==========================================
 *
 * Project Info:  https://github.com/jkinable/jorlib
 * Project Creator:  Joris Kinable (https://github.com/jkinable)
 *
 * (C) Copyright 2015, by Joris Kinable and Contributors.
 *
 * This program and the accompanying materials are licensed under LGPLv2.1
 *
 */
/* -----------------
 * FinishMasterEvent.java
 * -----------------
 * (C) Copyright 2015, by Joris Kinable and Contributors.
 *
 * Original Author:  Joris Kinable
 * Contributor(s):   -
 *
 * $Id$
 *
 * Changes
 * -------
 *
 */
package org.jorlib.frameworks.columnGeneration.branchAndPrice.EventHandling;

import org.jorlib.frameworks.columnGeneration.colgenMain.AbstractColumn;

import java.util.EventObject;
import java.util.List;

/**
 * Event generated when the column generation procedure finishes the pricing procedure
 *
 * @author Joris Kinable
 * @version 5-5-2015
 */
public class FinishPricingEvent extends EventObject{

    /** Indicates which iteration we are at in the column generation procedure **/
    public final int columnGenerationIteration;
    /** List of new columns, or empty list if no new columns could be found by the pricing problem**/
    public List<?  extends AbstractColumn<?, ?>> columns;
    /** Objective value of the master problem **/
    public final double objective;
    /** Cutoff value: Column Generation is terminated when the bound on the Master Objective is worse than the cutoff value **/
    public final int cutoffValue;
    /** Best available bound on the master objective **/
    public final double boundOnMasterObjective;

    /**
     * Creates a new FinishMasterEvent
     * @param source Generator of the event
     * @param columnGenerationIteration column generation iteration during which this event was fired
     * @param columns columns generated by the pricing problem
     * @param objective objective value
     * @param cutoffValue cutoff value
     * @param boundOnMasterObjective best available bound on the master objective
     * @param <U> type of column
     */
    public <U  extends AbstractColumn<?, ?>> FinishPricingEvent(Object source, int columnGenerationIteration, List<U> columns, double objective, int cutoffValue, double boundOnMasterObjective){
        super(source);
        this.columnGenerationIteration=columnGenerationIteration;
        this.columns=columns;
        this.objective=objective;
        this.cutoffValue=cutoffValue;
        this.boundOnMasterObjective=boundOnMasterObjective;
    }
}
